#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import threading
from threading import Event
from driver.AD7124 import Ad7124

from platform import system

_logger = logging.getLogger(__name__)


class AnalogInput(threading.Thread):
    """!
    Klasse zum Initialisieren des AnalogInput-Threads, Einlesen der Analogen Eingänge.
    Die Eingänge sind bereits durch die AD7124 A/D-gewandelt.
    Beispielsweise:
            Temperatursensor S: _temp_thermocouple_one
            Temperatursensor K: _temp_thermocouple_three
    """

    def __init__(self, error_handler, config_manager):
        """!
        Konstruktor der Klasse AnalogInput
        @param error_handler: Instanz des Errorhandlers
        @param config_manager: Instanz des Konfig-Managers
        """
        threading.Thread.__init__(self)
        self._ad7124 = Ad7124()
        self._error_handler = error_handler
        self._name = 'AnalogInput'
        self._thermocouple_error = 0x0000

        self._process_time = 0.1
        self._stopped = Event()
        self._stopped.clear()

        # Im Config-File stehen die Abgleich-Werte für die Drucksensoren
        self._ps1_offset = float(config_manager.get_calibration_file_value('pressure_sensors', 'ps1_offset', '0'))
        self._ps2_offset = float(config_manager.get_calibration_file_value('pressure_sensors', 'ps2_offset', '0'))
        self._ps3_offset = float(config_manager.get_calibration_file_value('pressure_sensors', 'ps3_offset', '0'))
        self._ps3_gradient = float(config_manager.get_calibration_file_value('pressure_sensors', 'ps3_gradient', '1'))
        self._corr_fact_s = float(config_manager.get_calibration_file_value('temperature_sensors', 'corr_fact_s', '1'))

        self._temp_cold_junction = self._ad7124.read_cold_junction()
        self._temp_thermocouple_one = self._ad7124.read_thermocouple_one() * self._corr_fact_s        # Korrekturfaktor anwenden
        self._temp_thermocouple_three = self._ad7124.read_thermocouple_three()
        self._pressure_sensor_1 = self._ad7124.read_pressure_sensor_one() + self._ps1_offset
        self._pressure_sensor_2 = self._ad7124.read_pressure_sensor_two() + self._ps2_offset
        ps3_temp = self._ad7124.read_pressure_sensor_three()
        self._pressure_sensor_3 = self._ps3_gradient * ps3_temp + self._ps3_offset      # Abgleich mittels Steigung und Offset

    def exit(self):
        """!
        Beendet den Thread.
        """
        self._stopped.set()

    @property
    def error_tc_s(self):
        """!
        Zustand von Temperatursensor S
        @return True - Fehler in Temperatursensor S (z.B.: Kabelbruch)
        """
        return self._thermocouple_error & self._ad7124.err_thermocouple_1

    @property
    def error_tc_k(self):
        """!
        Zustand von Temperatursensor K
        @return True - Fehler in Temperatursensor K (z.B.: Kabelbruch)
        """
        return self._thermocouple_error & self._ad7124.err_thermocouple_3

    @property
    def temp_elo_filtered(self):
        """!
        @return ELO (Hardware Elektronik) Temperatur [K]
        """
        return self._temp_cold_junction

    @property
    def temp_cold_junction_filtered(self):
        """!
        @return Wert der Cold-Junction Temperatur [K]
        """
        return self._temp_cold_junction

    @property
    def temp_tc_one_filtered(self):
        """!
        @return Temperatur an Sensor S (TC1) [K]
        """
        return self._temp_thermocouple_one

    @property
    def temp_tc_three_filtered(self):
        """!
        @return Temperatur an Sensor K [K]
        """
        return self._temp_thermocouple_three

    @property
    def pressure_sensor_1(self):
        """!
        Druck an Drucksensor 1
        @return Wert des Drucksensor 1[mbar]
        """
        return self._pressure_sensor_1

    @property
    def pressure_sensor_2(self):
        """!
        Druck an Drucksensor 2
        @return Wert des Drucksensor 2[mbar]
        """
        return self._pressure_sensor_2

    @property
    def pressure_sensor_3(self):
        """!
        Druck an Drucksensor 3
        @return Wert des Drucksensor 3 [mbar]
        """
        return self._pressure_sensor_3

    def run(self):
        """!
        Überschreibt die run()-Funktion der Bibliothek Treads um eine Spezifizierung der Aufgabe der
        abgeleiteten Klasse zu ermöglichen.
        Alle 100ms Abfrage der Temperaturwerte an Sensoren S und K, Filterung und Fehlerüberwachung
        """
        meas_state = 0
        while not self._stopped.wait(self._process_time):
            # ELO
            self._temp_cold_junction = self._ad7124.read_cold_junction()
            # type S
            self._temp_thermocouple_one = self._ad7124.read_thermocouple_one() * self._corr_fact_s        # Korrekturfaktor anwenden
            # type K
            self._temp_thermocouple_three = self._ad7124.read_thermocouple_three()
            # pressure sensor 1
            self._pressure_sensor_1 = self._ad7124.read_pressure_sensor_one() + self._ps1_offset
            # pressure sensor 2
            self._pressure_sensor_2 = self._ad7124.read_pressure_sensor_two() + self._ps2_offset
            # pressure sensor 3
            ps3_temp = self._ad7124.read_pressure_sensor_three()
            self._pressure_sensor_3 = self._ps3_gradient * ps3_temp + self._ps3_offset      # Abgleich mittels Steigung und Offset

            if meas_state >= 4:
                if system() == 'Windows':
                    pass    # TODO: Emulator-Testfunktion für Fühlerbruch implementieren
                else:
                    # check error thermocouple 1
                    adc_val = self._ad7124.read_channel(self._ad7124.dev_ad7124.AD7124_Channel_3)

                    # if self._temp_thermocouple_one == 0:
                    if adc_val == 0:
                        self._thermocouple_error |= self._ad7124.err_thermocouple_1
                        self._error_handler.set_error("F40")    # S-Element Kurzschluss
                    elif adc_val > self._ad7124.channel_offset + self._ad7124.adc_val_sens_break:
                        self._thermocouple_error |= self._ad7124.err_thermocouple_1
                        self._error_handler.set_error("F41")    # S-Element Bruch
                    else:
                        self._thermocouple_error &= ~self._ad7124.err_thermocouple_1

                    # check error thermocouple 3
                    adc_val = self._ad7124.read_channel(self._ad7124.dev_ad7124.AD7124_Channel_4)

                    # if self._temp_thermocouple_three == 0:
                    if adc_val == 0:
                        self._thermocouple_error |= self._ad7124.err_thermocouple_3
                        self._error_handler.set_error("F400")   # K-Element Kurzschluss
                    elif adc_val > self._ad7124.channel_offset + self._ad7124.adc_val_sens_break:
                        self._thermocouple_error |= self._ad7124.err_thermocouple_3
                        self._error_handler.set_error("F401")   # K-Element Bruch
                    else:
                        self._thermocouple_error &= ~self._ad7124.err_thermocouple_3

                    if self._thermocouple_error is not 0:
                        _logger.error("thermocouple_error: {}".format(self._thermocouple_error))

                meas_state = 0

            meas_state += 1
