#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

_logger = logging.getLogger(__name__)
ENABLE_DEBUG_LOG = False


class VacuumControl(object):
    """!
    Unterdruckerzeugung und Unterdruckregelung
    """
    def __init__(self, daio, analog_input, config_manager):
        """!
        Initialisiert die Unterdruckerzeugung und den Regler.
        @param daio: Instanz der DIO
        @param analog_input: Instanz der AIO
        """
        _logger.debug("VacuumControl init")
        self._name = "VacuumControl"

        self._daio = daio
        self._analog_input = analog_input
        self._enable_vacuum = False

        # Steigt der Druck in der Brennkammer über diesen Wert, schaltet die Pumpe an
        self._vacuum_max = int(config_manager.get_config_file_value('vacuum_control', 'vacuum_max'))
        # Fällt der Druck in der Brennkammer unter diesen Wert, besteht Unterdruck
        self._vacuum_min = int(config_manager.get_config_file_value('vacuum_control', 'vacuum_min'))

        self.disableVacuum()

    def enableVacuum(self):
        """!
        Unterdruckerzeugung und -regelung aktivieren
        """
        self._enable_vacuum = True
        vacuum_pressure = self._analog_input.pressure_sensor_3          # Drucksensor 3 lesen
        if vacuum_pressure > self._vacuum_max:                       # Zu wenig Unterdruck
            self._daio.rel_vakuum(True)                                 # Pumpe ein
            self._daio.valve_vacuum(False)                              # Unterdruck-Ventil öffnet, wenn Strom kein fließt

    def disableVacuum(self):
        """!
        Unterdruckerzeugung und -regelung deaktivieren
        """
        self._enable_vacuum = False
        self._daio.rel_vakuum(False)                                    # Punpe aus
        self._daio.valve_vacuum(False)                                  # Unterdruck-Ventil offen (kein Strom fließt)

    def __call__(self):
        """!
        Unterdruckregelung, muss zyklisch aufgerufen werden.
        """
        # Unterdruckerzeugung in der Brennkammer ein
        if self._enable_vacuum:                                         # Unterdruck gewünscht
            vacuum_pressure = self._analog_input.pressure_sensor_3      # Drucksensor 3 lesen
            if vacuum_pressure > self._vacuum_max:                      # Zu wenig Unterdruck
                self._daio.rel_vakuum(True)                             # Pumpe ein
                self._daio.valve_vacuum(False)                          # Unterdruck-Ventil öffnet, wenn Strom kein fließt
            if vacuum_pressure < self._vacuum_min:                      # Unterdruck ausreichend
                self._daio.rel_vakuum(False)                            # Pumpe aus
                self._daio.valve_vacuum(True)                           # Unterdruck-Ventil schließt, wenn Strom fließt
        # Unterdruckerzeugung in der Brennkammer aus
        else:                                                           # Kein Unterdruck gewünscht
            self._daio.rel_vakuum(False)                                # Pumpe aus
            self._daio.valve_vacuum(False)                              # Unterdruck-Ventil offen (kein Strom fließt)
